package CatalystX::Features::Plugin::Init;
our $VERSION = '0.1011';


=head1 NAME

CatalystX::Features::Plugin::Init - Let the feature run initialization code

=head1 VERSION

version 0.1011

=head1 DESCRIPTION

WIP. Doesn't do anything just yet.

=head1 AUTHORS

	Rodrigo de Oliveira (rodrigolive), C<rodrigolive@gmail.com>

=head1 LICENSE

This library is free software. You can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
1;